@@ -13,13 +13,14 @@ Unless required by applicable law or agreed to in writing, software distributed
  language governing permissions and limitations under the License.
 """
 
-from flask import abort, request, send_file, send_from_directory, render_template, render_template_string, \
-    url_for
-import werkzeug.exceptions as exceptions
-from .app import App
-from .api import Api
-from .problem import problem
-from .decorators.produces import NoContent
-from .resolver import Resolution, Resolver, RestyResolver
+from flask import (abort, request, send_file, send_from_directory,  # NOQA
+                   render_template, render_template_string, url_for)
+import werkzeug.exceptions as exceptions  # NOQA
+from .app import App  # NOQA
+from .api import Api  # NOQA
+from .problem import problem  # NOQA
+from .decorators.produces import NoContent  # NOQA
+from .resolver import Resolution, Resolver, RestyResolver  # NOQA
 
-__version__ = '1.0.81'
+# This version is replaced during release process.
+__version__ = '1.0.89'
