@@ -11,45 +11,74 @@ Unless required by applicable law or agreed to in writing, software distributed
  language governing permissions and limitations under the License.
 """
 
-import flask
-import jinja2
 import json
 import logging
 import pathlib
+import sys
+
+import flask
+import jinja2
+import six
+import werkzeug.exceptions
 import yaml
+from swagger_spec_validator.validator20 import validate_spec
+
+from . import resolver, utils
+from .handlers import AuthErrorHandler
 from .operation import Operation
-from . import utils
 
 MODULE_PATH = pathlib.Path(__file__).absolute().parent
-SWAGGER_UI_PATH = MODULE_PATH / 'swagger-ui'
+SWAGGER_UI_PATH = MODULE_PATH / 'vendor' / 'swagger-ui'
 SWAGGER_UI_URL = 'ui'
 
 logger = logging.getLogger('connexion.api')
 
 
-class Api:
+def compatibility_layer(spec):
+    """Make specs compatible with older versions of Connexion."""
+    # Make all response codes be string
+    for path_name, methods_available in spec.get('paths', {}).items():
+        for method_name, method_def in methods_available.items():
+            if method_name == 'parameters':
+                continue
+            response_definitions = {}
+            for response_code, response_def in method_def.get('responses', {}).items():
+                response_definitions[str(response_code)] = response_def
+                method_def['responses'] = response_definitions
+    return spec
+
+
+class Api(object):
     """
     Single API that corresponds to a flask blueprint
     """
 
-    def __init__(self, swagger_yaml_path, base_url=None, arguments=None, swagger_ui=None, swagger_path=None,
-                 swagger_url=None, validate_responses=False, resolver=utils.get_function_from_name):
+    def __init__(self, swagger_yaml_path, base_url=None, arguments=None,
+                 swagger_json=None, swagger_ui=None, swagger_path=None, swagger_url=None,
+                 validate_responses=False, resolver=resolver.Resolver(),
+                 auth_all_paths=False, debug=False):
         """
         :type swagger_yaml_path: pathlib.Path
         :type base_url: str | None
         :type arguments: dict | None
+        :type swagger_json: bool
         :type swagger_ui: bool
         :type swagger_path: string | None
         :type swagger_url: string | None
+        :type auth_all_paths: bool
+        :type debug: bool
         :param resolver: Callable that maps operationID to a function
         """
+        self.debug = debug
         self.swagger_yaml_path = pathlib.Path(swagger_yaml_path)
-        logger.debug('Loading specification: %s', swagger_yaml_path, extra={'swagger_yaml': swagger_yaml_path,
-                                                                            'base_url': base_url,
-                                                                            'arguments': arguments,
-                                                                            'swagger_ui': swagger_ui,
-                                                                            'swagger_path': swagger_path,
-                                                                            'swagger_url': swagger_url})
+        logger.debug('Loading specification: %s', swagger_yaml_path,
+                     extra={'swagger_yaml': swagger_yaml_path,
+                            'base_url': base_url,
+                            'arguments': arguments,
+                            'swagger_ui': swagger_ui,
+                            'swagger_path': swagger_path,
+                            'swagger_url': swagger_url,
+                            'auth_all_paths': auth_all_paths})
         arguments = arguments or {}
         with swagger_yaml_path.open() as swagger_yaml:
             swagger_template = swagger_yaml.read()
@@ -58,6 +87,8 @@ class Api:
 
         logger.debug('Read specification', extra=self.specification)
 
+        validate_spec(compatibility_layer(self.specification))
+
         # https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#fixed-fields
         # If base_url is not on provided then we try to read it from the swagger.yaml or use / by default
         if base_url is None:
@@ -76,6 +107,7 @@ class Api:
 
         self.definitions = self.specification.get('definitions', {})
         self.parameter_definitions = self.specification.get('parameters', {})
+        self.response_definitions = self.specification.get('responses', {})
 
         self.swagger_path = swagger_path or SWAGGER_UI_PATH
         self.swagger_url = swagger_url or SWAGGER_UI_URL
@@ -88,12 +120,17 @@ class Api:
         # Create blueprint and endpoints
         self.blueprint = self.create_blueprint()
 
-        self.add_swagger_json()
+        if swagger_json:
+            self.add_swagger_json()
         if swagger_ui:
             self.add_swagger_ui()
+
         self.add_paths()
 
-    def add_operation(self, method, path, swagger_operation):
+        if auth_all_paths:
+            self.add_auth_on_not_found()
+
+    def add_operation(self, method, path, swagger_operation, path_parameters):
         """
         Adds one operation to the api.
 
@@ -110,15 +147,24 @@ class Api:
         :type path: str
         :type swagger_operation: dict
         """
-        operation = Operation(method=method, path=path, operation=swagger_operation,
-                              app_produces=self.produces, app_security=self.security,
-                              security_definitions=self.security_definitions, definitions=self.definitions,
+        operation = Operation(method=method,
+                              path=path,
+                              path_parameters=path_parameters,
+                              operation=swagger_operation,
+                              app_produces=self.produces,
+                              app_security=self.security,
+                              security_definitions=self.security_definitions,
+                              definitions=self.definitions,
                               parameter_definitions=self.parameter_definitions,
-                              validate_responses=self.validate_responses, resolver=self.resolver)
+                              response_definitions=self.response_definitions,
+                              validate_responses=self.validate_responses,
+                              resolver=self.resolver)
         operation_id = operation.operation_id
-        logger.debug('... Adding %s -> %s', method.upper(), operation_id, extra=vars(operation))
+        logger.debug('... Adding %s -> %s', method.upper(), operation_id,
+                     extra=vars(operation))
 
-        self.blueprint.add_url_rule(path, operation.endpoint_name, operation.function, methods=[method])
+        flask_path = utils.flaskify_path(path, operation.get_path_parameter_types())
+        self.blueprint.add_url_rule(flask_path, operation.endpoint_name, operation.function, methods=[method])
 
     def add_paths(self, paths=None):
         """
@@ -129,13 +175,38 @@ class Api:
         paths = paths or self.specification.get('paths', dict())
         for path, methods in paths.items():
             logger.debug('Adding %s%s...', self.base_url, path)
-            path = utils.flaskify_path(path)
+
+            # search for parameters definitions in the path level
+            # http://swagger.io/specification/#pathItemObject
+            path_parameters = methods.get('parameters', [])
+
             # TODO Error handling
             for method, endpoint in methods.items():
+                if method == 'parameters':
+                    continue
                 try:
-                    self.add_operation(method, path, endpoint)
+                    self.add_operation(method, path, endpoint, path_parameters)
                 except Exception:  # pylint: disable= W0703
-                    logger.exception('Failed to add operation for %s %s%s', method.upper(), self.base_url, path)
+                    url = '{base_url}{path}'.format(base_url=self.base_url,
+                                                    path=path)
+                    error_msg = 'Failed to add operation for {method} {url}'.format(
+                        method=method.upper(),
+                        url=url)
+                    if self.debug:
+                        logger.exception(error_msg)
+                    else:
+                        logger.error(error_msg)
+                        six.reraise(*sys.exc_info())
+
+    def add_auth_on_not_found(self):
+        """
+        Adds a 404 error handler to authenticate and only expose the 404 status if the security validation pass.
+        """
+        logger.debug('Adding path not found authentication')
+        not_found_error = AuthErrorHandler(werkzeug.exceptions.NotFound(), security=self.security,
+                                           security_definitions=self.security_definitions)
+        endpoint_name = "{name}_not_found".format(name=self.blueprint.name)
+        self.blueprint.add_url_rule('/<path:invalid_path>', endpoint_name, not_found_error.function)
 
     def add_swagger_json(self):
         """
