@@ -13,12 +13,15 @@ Unless required by applicable law or agreed to in writing, software distributed
 
 import logging
 import pathlib
+
 import flask
 import werkzeug.exceptions
-from .problem import problem
-from .api import Api
+from connexion.decorators.produces import JSONEncoder as ConnexionJSONEncoder
 from connexion.resolver import Resolver
 
+from .api import Api
+from .problem import problem
+
 logger = logging.getLogger('connexion.app')
 
 
@@ -53,6 +56,8 @@ class App(object):
         """
         self.app = flask.Flask(import_name)
 
+        self.app.json_encoder = ConnexionJSONEncoder
+
         # we get our application root path from flask to avoid duplicating logic
         self.root_path = pathlib.Path(self.app.root_path)
         logger.debug('Root Path: %s', self.root_path)
