@@ -1,12 +1,13 @@
-import werkzeug.exceptions as exceptions
 import copy
-import flask
 import functools
 import inspect
 import logging
-import six
 
-from ..utils import boolean
+import flask
+import six
+import werkzeug.exceptions as exceptions
+
+from ..utils import boolean, is_null, is_nullable
 
 logger = logging.getLogger(__name__)
 
@@ -38,6 +39,9 @@ def make_type(value, type):
 
 
 def get_val_from_param(value, query_param):
+    if is_nullable(query_param) and is_null(value):
+        return None
+
     if query_param["type"] == "array":  # then logic is more complex
         if query_param.get("collectionFormat") and query_param.get("collectionFormat") == "pipes":
             parts = value.split("|")
@@ -92,12 +96,11 @@ def parameter_to_arg(parameters, function):
                                                  path_param_definitions)
 
         # Add body parameters
-        if request_body is not None:
-            if body_name not in arguments:
-                logger.debug("Body parameter '%s' not in function arguments", body_name)
-            else:
-                logger.debug("Body parameter '%s' in function arguments", body_name)
-                kwargs[body_name] = request_body
+        if body_name not in arguments:
+            logger.debug("Body parameter '%s' not in function arguments", body_name)
+        else:
+            logger.debug("Body parameter '%s' in function arguments", body_name)
+            kwargs[body_name] = request_body
 
         # Add query parameters
         query_arguments = copy.deepcopy(default_query_params)
@@ -130,6 +133,15 @@ def parameter_to_arg(parameters, function):
                 else:
                     kwargs[key] = get_val_from_param(value, form_param)
 
+        # Add file parameters
+        file_arguments = flask.request.files
+        for key, value in file_arguments.items():
+            if key not in arguments:
+                logger.debug("File parameter (formData) '%s' not in function arguments", key)
+            else:
+                logger.debug("File parameter (formData) '%s' in function arguments", key)
+                kwargs[key] = value
+
         return function(*args, **kwargs)
 
     return wrapper
